import { Config, PathItem, DBConfig, Entity, JsonObject, IStorageAdapter } from '../types';
import { Cache } from '../internal';
export declare class NearDB {
    /** Config that is used to init NearDB */
    config: DBConfig;
    /** UUID of Instance of NearDB */
    instanceId: string;
    adapter: IStorageAdapter;
    static field: {
        deleteValue: string;
    };
    /**
     * Constructor to setup config, and path, and required checks.
     * @param config configuration to initialize NearDB instance
     */
    constructor(config: Config);
    /**
     * Static method to create a new instance of NearDB class.
     * @param config configuration to initialize NearDB instance
     * @returns an initialized instance of NearDB with the config
     */
    static database(config: Config): NearDB;
    /**
     * Returns a new instance of collection
     * @param key expects key for collection
     * @returns an instance of the collection class
     */
    collection(key: string): Collection;
}
export declare class Collection implements Entity {
    readonly uri: PathItem[];
    readonly instance: NearDB;
    constructor(instance: NearDB, key: string, path?: PathItem[]);
    doc(key: string): Document;
    /**
     * Adds a document to a collection by generating an id for the doc
     * @param value expects payload to be stored for the document
     * @returns a promise for the payload of the saved doc
     */
    add(value: JsonObject): Promise<string>;
}
export declare class Document implements Entity {
    /** Data path that is used to interact with storage */
    readonly uri: PathItem[];
    /** Instance of NearDB with all the configuration and env settings */
    readonly instance: NearDB;
    /** Offline cache of data */
    readonly cache: Cache;
    /** String path for storage */
    constructor(instance: NearDB, key: string, path: PathItem[]);
    /**
     * Creates a new instance of Collection
     * @param key expects key for collection
     * @returns an instance of the collection class
     */
    collection(key: string): Collection;
    /**
     * Sets document data from the path provided in the scope.
     * @param value expects payload to be stored for the document
     * @returns payload of the document requested
     */
    set(value: JsonObject): Promise<string>;
    /**
     * Gets document data from the path provided in the scope.
     * @param options sets options on how to get documents
     * @returns payload of the document requested
     */
    get(): Promise<JsonObject | null>;
    /**
     * Update some fields of the document without overwriting entire document
     * @param value expects payload to be stored for the document
     * @returns a promise for the payload requested
     */
    update(value: JsonObject): Promise<void>;
    /**
     * Removes document store in the path of instance
     * @returns payload of the document requested
     */
    delete(): Promise<void>;
}
